@@ -16,10 +16,10 @@ package load
 
 import (
 	"cmp"
+	"maps"
 	pathpkg "path"
 	"path/filepath"
 	"slices"
-	"sort"
 	"strconv"
 	"strings"
 
@@ -147,7 +147,7 @@ func (fp *fileProcessor) finalize(p *build.Instance) errors.Error {
 		return fp.err
 	}
 
-	p.ImportPaths, _ = cleanImports(fp.imported)
+	p.ImportPaths = slices.Sorted(maps.Keys(fp.imported))
 
 	return nil
 }
@@ -345,15 +345,6 @@ func (fp *fileProcessor) add(root string, file *build.File, mode importMode) {
 	}
 }
 
-func cleanImports(m map[string][]token.Pos) ([]string, map[string][]token.Pos) {
-	all := make([]string, 0, len(m))
-	for path := range m {
-		all = append(all, path)
-	}
-	sort.Strings(all)
-	return all, m
-}
-
 // isLocalImport reports whether the import path is
 // a local import path, like ".", "..", "./foo", or "../foo".
 func isLocalImport(path string) bool {
