@@ -22,6 +22,7 @@ package modfile
 import (
 	_ "embed"
 	"fmt"
+	"path"
 	"slices"
 	"strings"
 	"sync"
@@ -48,12 +49,18 @@ const schemaFile = "cuelang.org/go/mod/modfile/schema.cue"
 
 // File represents the contents of a cue.mod/module.cue file.
 type File struct {
-	Module   string                    `json:"module"`
-	Language *Language                 `json:"language,omitempty"`
-	Source   *Source                   `json:"source,omitempty"`
-	Deps     map[string]*Dep           `json:"deps,omitempty"`
-	Custom   map[string]map[string]any `json:"custom,omitempty"`
-	versions []module.Version
+	// Module holds the module path, which may
+	// not contain a major version suffix.
+	// Use the [File.QualifiedModule] method to obtain a module
+	// path that's always qualified. See also the
+	// [File.ModulePath] and [File.MajorVersion] methods.
+	Module          string                    `json:"module"`
+	Language        *Language                 `json:"language,omitempty"`
+	Source          *Source                   `json:"source,omitempty"`
+	Deps            map[string]*Dep           `json:"deps,omitempty"`
+	Custom          map[string]map[string]any `json:"custom,omitempty"`
+	versions        []module.Version
+	versionByModule map[string]module.Version
 	// defaultMajorVersions maps from module base path to the
 	// major version default for that path.
 	defaultMajorVersions map[string]string
@@ -64,6 +71,39 @@ type File struct {
 	actualSchemaVersion string
 }
 
+// Module returns the fully qualified module path
+// if is one. It returns the empty string when [ParseLegacy]
+// is used and the module field is empty.
+//
+// Note that when the module field does not contain a major
+// version suffix, "@v0" is assumed.
+func (f *File) QualifiedModule() string {
+	if strings.Contains(f.Module, "@") {
+		return f.Module
+	}
+	if f.Module == "" {
+		return ""
+	}
+	return f.Module + "@v0"
+}
+
+// ModulePath returns the path part of the module without
+// its major version suffix.
+func (f *File) ModulePath() string {
+	path, _, _ := ast.SplitPackageVersion(f.QualifiedModule())
+	return path
+}
+
+// MajorVersion returns the major version of the module,
+// not including the "@".
+// If there is no module (which can happen when [ParseLegacy]
+// is used or if Module is explicitly set to an empty string),
+// it returns the empty string.
+func (f *File) MajorVersion() string {
+	_, vers, _ := ast.SplitPackageVersion(f.QualifiedModule())
+	return vers
+}
+
 // baseFileVersion is used to decode the language version
 // to decide how to decode the rest of the file.
 type baseFileVersion struct {
@@ -118,7 +158,7 @@ func (f *File) Format() ([]byte, error) {
 	// before formatting the output.
 	f1, err := ParseNonStrict(data, "-")
 	if err != nil {
-		return nil, fmt.Errorf("cannot round-trip module file: %v", strings.TrimSuffix(errors.Details(err, nil), "\n"))
+		return nil, fmt.Errorf("cannot parse result: %v", strings.TrimSuffix(errors.Details(err, nil), "\n"))
 	}
 	if f.Language != nil && f1.actualSchemaVersion == "v0.0.0" {
 		// It's not a legacy module file (because the language field is present)
@@ -194,30 +234,20 @@ func lookup(v cue.Value, sels ...cue.Selector) cue.Value {
 // should be at least this, because that's when we added the language.version
 // field itself.
 func EarliestClosedSchemaVersion() string {
-	return schemaVersionLimits()[0]
+	return earliestClosedSchemaVersion()
 }
 
-// LatestKnownSchemaVersion returns the language version
-// associated with the most recent known schema.
-func LatestKnownSchemaVersion() string {
-	return schemaVersionLimits()[1]
-}
-
-var schemaVersionLimits = sync.OnceValue(func() [2]string {
-	limits, _ := moduleSchemaDo(func(info *schemaInfo) ([2]string, error) {
+var earliestClosedSchemaVersion = sync.OnceValue(func() string {
+	earliest, _ := moduleSchemaDo(func(info *schemaInfo) (string, error) {
 		earliest := ""
-		latest := ""
 		for v := range info.Versions {
 			if earliest == "" || semver.Compare(v, earliest) < 0 {
 				earliest = v
 			}
-			if latest == "" || semver.Compare(v, latest) > 0 {
-				latest = v
-			}
 		}
-		return [2]string{earliest, latest}, nil
+		return earliest, nil
 	})
-	return limits
+	return earliest
 })
 
 // Parse verifies that the module file has correct syntax
@@ -266,7 +296,7 @@ func ParseNonStrict(modfile []byte, filename string) (*File, error) {
 // into a format suitable for parsing with [Parse]. It adds a language.version
 // field and moves all unrecognized fields into custom.legacy.
 //
-// If there is no module field or it is empty, it is set to "test.example@v0".
+// If there is no module field or it is empty, it is set to "test.example".
 //
 // If the file already parses OK with [ParseNonStrict], it returns the
 // result of that.
@@ -289,7 +319,7 @@ func FixLegacy(modfile []byte, filename string) (*File, error) {
 	if err := v.Decode(&allFields); err != nil {
 		return nil, err
 	}
-	mpath := "test.example@v0"
+	mpath := "test.example"
 	if m, ok := allFields["module"]; ok {
 		if mpath1, ok := m.(string); ok && mpath1 != "" {
 			mpath = mpath1
@@ -313,7 +343,12 @@ func FixLegacy(modfile []byte, filename string) (*File, error) {
 	f = &File{
 		Module: mpath,
 		Language: &Language{
-			Version: cueversion.LanguageVersion(),
+			// If there's a legacy module file, the CUE code
+			// is unlikely to be using new language features,
+			// so keep the language version fixed rather than
+			// using [cueversion.LanguageVersion].
+			// See https://cuelang.org/issue/3222.
+			Version: "v0.9.0",
 		},
 		Custom: custom,
 	}
@@ -326,7 +361,7 @@ func FixLegacy(modfile []byte, filename string) (*File, error) {
 	}
 	f, err = ParseNonStrict(data, "fixed-"+filename)
 	if err != nil {
-		return nil, fmt.Errorf("cannot round-trip fixed module file %q: %v", data, err)
+		return nil, fmt.Errorf("cannot parse resulting module file %q: %v", data, err)
 	}
 	return f, nil
 }
@@ -407,22 +442,19 @@ func parse(modfile []byte, filename string, strict bool) (*File, error) {
 	if err != nil {
 		return nil, err
 	}
-	mainPath, mainMajor, ok := module.SplitPathVersion(mf.Module)
-	if strict && !ok {
-		return nil, fmt.Errorf("module path %q in %s does not contain major version", mf.Module, filename)
-	}
+	mainPath, mainMajor, ok := ast.SplitPackageVersion(mf.Module)
 	if ok {
 		if semver.Major(mainMajor) != mainMajor {
 			return nil, fmt.Errorf("module path %s in %q should contain the major version only", mf.Module, filename)
 		}
-	} else if mainPath = mf.Module; mainPath != "" {
+	} else if mainPath != "" {
 		if err := module.CheckPathWithoutVersion(mainPath); err != nil {
 			return nil, fmt.Errorf("module path %q in %q is not valid: %v", mainPath, filename, err)
 		}
 		// There's no main module major version: default to v0.
 		mainMajor = "v0"
-		// TODO perhaps we'd be better preserving the original?
-		mf.Module += "@v0"
+	} else {
+		return nil, fmt.Errorf("empty module path in %q", filename)
 	}
 	if mf.Language != nil {
 		vers := mf.Language.Version
@@ -433,10 +465,12 @@ func parse(modfile []byte, filename string, strict bool) (*File, error) {
 			return nil, fmt.Errorf("language version %v in %s is not canonical", vers, filename)
 		}
 	}
+	mf.versionByModule = make(map[string]module.Version)
 	var versions []module.Version
-	// The main module is always the default for its own major version.
-	defaultMajorVersions := map[string]string{
-		mainPath: mainMajor,
+	defaultMajorVersions := make(map[string]string)
+	if mainPath != "" {
+		// The main module is always the default for its own major version.
+		defaultMajorVersions[mainPath] = mainMajor
 	}
 	// Check that major versions match dependency versions.
 	for m, dep := range mf.Deps {
@@ -455,13 +489,22 @@ func parse(modfile []byte, filename string, strict bool) (*File, error) {
 			}
 			defaultMajorVersions[mp] = semver.Major(vers.Version())
 		}
+		mf.versionByModule[vers.Path()] = vers
+	}
+	if mainPath != "" {
+		// We don't necessarily have a full version for the main module.
+		mainWithMajor := mainPath + "@" + mainMajor
+		mainVersion, err := module.NewVersion(mainWithMajor, "")
+		if err != nil {
+			return nil, err
+		}
+		mf.versionByModule[mainWithMajor] = mainVersion
 	}
-
 	if len(defaultMajorVersions) > 0 {
 		mf.defaultMajorVersions = defaultMajorVersions
 	}
 	mf.versions = versions[:len(versions):len(versions)]
-	module.Sort(mf.versions)
+	slices.SortFunc(mf.versions, module.Version.Compare)
 	return mf, nil
 }
 
@@ -551,3 +594,27 @@ func (f *File) DepVersions() []module.Version {
 func (f *File) DefaultMajorVersions() map[string]string {
 	return f.defaultMajorVersions
 }
+
+// ModuleForImportPath returns the module that should contain the given
+// import path and reports whether the module was found.
+// It does not check to see if the import path actually exists within the module.
+//
+// It works entirely from information in f, meaning that it does
+// not consult a registry to resolve a package whose module is not
+// mentioned in the file, which means it will not work in general unless
+// the module is tidy (as with `cue mod tidy`).
+func (f *File) ModuleForImportPath(importPath string) (module.Version, bool) {
+	ip := ast.ParseImportPath(importPath)
+	for prefix := ip.Path; prefix != "."; prefix = path.Dir(prefix) {
+		pkgVersion := ip.Version
+		if pkgVersion == "" {
+			if pkgVersion = f.defaultMajorVersions[prefix]; pkgVersion == "" {
+				continue
+			}
+		}
+		if mv, ok := f.versionByModule[prefix+"@"+pkgVersion]; ok {
+			return mv, true
+		}
+	}
+	return module.Version{}, false
+}
